<?php
/**
 * A Magento 2 module named FiloBlu/Cegid
 * Copyright (C) 2019  2019
 *
 * This file is part of FiloBlu/Cegid.
 *
 * FiloBlu/Cegid is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

namespace FiloBlu\Cegid\Console\Command;

use Exception;
use FiloBlu\Cegid\Helper\Cegid\Customer;
use Magento\Customer\Api\CustomerRepositoryInterface;
use Magento\Framework\App\ResourceConnection;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Zend_DB;

/**
 * Class Import
 * @package FiloBlu\Cegid\Console\Command
 */
class Synchronize extends Command
{
    /**
     * @var Customer
     */
    protected $customerHelper;
    /**
     * @var ResourceConnection
     */
    protected $resourceConnection;
    /**
     * @var CustomerRepositoryInterface
     */
    protected $customerRepository;

    /**
     * Import constructor.
     * @param Customer $customerHelper
     * @param CustomerRepositoryInterface $customerRepository
     * @param ResourceConnection $resourceConnection
     * @param string $name
     */
    public function __construct(Customer $customerHelper,
                                CustomerRepositoryInterface $customerRepository,
                                ResourceConnection $resourceConnection,
                                $name = null)
    {
        $this->customerHelper = $customerHelper;
        $this->resourceConnection = $resourceConnection;
        $this->customerRepository = $customerRepository;
        parent::__construct($name);
    }

    /**
     * {@inheritdoc}
     */
    protected function execute(
        InputInterface $input,
        OutputInterface $output
    )
    {

        $customers = $this->getCustomerToSyncronize();
        foreach ($customers as $customerInfo) {
            $id = $customerInfo['entity_id'];
            $output->write("Processing {$id} ... ");
            try {
                $customer = $this->customerRepository->getById($id);
                $found = $this->customerHelper->searchCustomerByEmail($customer);

                $response = $found->getSearchCustomerIdsResult()->getCustomerQueryData();


                if ($response === null) {
                    $this->customerHelper->createCustomer($customer);
                    $output->writeln('<info>Created</info>');

                } else {
                    if ($customer->getCustomAttribute('cegid_id') !== null) {
                        $output->writeln('<info>Skipped</info>');
                        continue;
                    }
                    $customer->setCustomAttribute('cegid_id', $response[0]->getCustomerId());
                    try {
                        $this->customerRepository->save($customer);
                        $output->writeln('<info>Created</info>');
                    } catch (Exception $e) {
                        $output->writeln($e->getMessage());
                    }
                }

            } catch (Exception $e) {
                $output->writeln("Customer with id = {$id} not found.");
            }


        }
    }

    /**
     * @return array
     */
    protected function getCustomerToSyncronize()
    {
        return $this->resourceConnection->getConnection()->fetchAll('SELECT entity_id FROM customer_entity WHERE entity_id NOT IN(
SELECT
    ce.entity_id
FROM
    customer_entity ce
INNER JOIN customer_entity_varchar cev ON
    ce.entity_id = cev.entity_id
INNER JOIN eav_attribute ea ON
    cev.attribute_id = ea.attribute_id AND ea.attribute_code  = \'cegid_id\'
    )', [], Zend_DB::FETCH_ASSOC);
    }

    /**
     * {@inheritdoc}
     */
    protected function configure()
    {
        $this->setName('filoblu:cegid:import');
        $this->setDescription('Syncronize Cegid id into Magento2 cusotmers');
        parent::configure();
    }
}
