<?php

namespace FiloBlu\Cegid\Helper\Cegid;
use Magento\Framework\App\Helper\AbstractHelper;
use \Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Sales\Api\Data\OrderInterface;

/**
 * Class Customer
 * @package FiloBlu\Cegid\Helper\Cegid
 */
class Base extends AbstractHelper
{

    /**
     * @param \Magento\Framework\App\Helper\Context $context
     */
    public function __construct(
        \Magento\Framework\App\Helper\Context $context
    ) {
        parent::__construct($context);
    }


    public function getAddressStreet($address,$index = null){
        $street_data = $address->getStreet();

        // If $street_data is a string
        if(is_string($street_data)){
            if ($index === null || $index == 1){
                return $street_data;
            } else {
                return "";
            }
        }

        // If $street_data is array
        if (is_array($street_data)){
            $result = array();
            $result[1] = (isset($street_data[0]) ? $street_data[0] : "");
            $result[2] = (isset($street_data[1]) ? $street_data[1] : "");
            $result[3] = (isset($street_data[2]) ? $street_data[2] : "");
            $return = ($index > 0 && isset($result[$index]) ? $result[$index] : "");
            return $return;
        }

        // Not array and not string
        return "";
    }

    // This function will work to retrieve custom customer attributes under Magento 2.2.x and 2.3.x
    public function getCompatibleCustomerCustomAttribute($customer,$custom_attribute_code){
        // This is for Magento 2.2.x
        $attribute_value = $customer->getCustomAttribute($custom_attribute_code);
        // Fix for Magento 2.3.x
        if (!is_string($attribute_value) && $attribute_value != null){
            $attribute_value = $customer->getCustomAttribute($custom_attribute_code)->getValue();
        }
        return $attribute_value;
    }

    public function getCegidPaymentMethodId(OrderInterface $order){

        // Manual Mapping
        // TODO: Extract this on a system configuration
        // https://docs.google.com/spreadsheets/d/1-cMb02veC81dAZ5-ARIs-U6kWEV6xg43z5qZzOv8mnA/edit?usp=sharing

        $method_id_parts = array();
        $method_id_parts[] = $order->getPayment()->getMethod();
        $method_id_parts[] = $order->getPayment()->getCcType();
        $method_id_parts[] = $order->getBaseCurrencyCode();

        $unique_method_id = implode('-',array_filter($method_id_parts));

        $fallback = 'E16';
        $mp = array(
            'adyen_cc-AE-CHF' => 'C01',
            'adyen_cc-DN-CHF' => 'C04',
            'adyen_cc-JCB-CHF' => 'C05',
            'adyen_cc-MI-CHF' => 'C06',
            'adyen_cc-MC-CHF' => 'C07',
            'adyen_cc-VI-CHF' => 'C08',
            'adyen_hpp-directEbanking-CHF' => 'C10',
            'adyen_cc-AE-EUR' => 'E01',
            'adyen_cc-UN-EUR' => 'E03',
            'adyen_cc-DN-EUR' => 'E04',
            'adyen_cc-JCB-EUR' => 'E05',
            'adyen_cc-MI-EUR' => 'E06',
            'adyen_cc-MC-EUR' => 'E07',
            'adyen_cc-VI-EUR' => 'E08',
            'adyen_hpp-bankTransfer_IBAN-EUR' => 'E10',
            'adyen_hpp-directEbanking-EUR' => 'E10',
            'msp_cashondelivery' => 'E11',
            'msp_cashondelivery-EUR' => 'E11',
            'adyen_cc-AE-USD' => 'U01',
            'adyen_cc-UN-USD' => 'U03',
            'adyen_cc-DN-USD' => 'U04',
            'adyen_cc-JCB-USD' => 'U05',
            'adyen_cc-MI-USD' => 'U06',
            'adyen_cc-MC-USD' => 'U07',
            'adyen_cc-VI-USD' => 'U08'
        );

        if ($unique_method_id && isset($mp[$unique_method_id])){
            return $mp[$unique_method_id];
        }

        return $fallback;
    }

    public function writeLog($method = null,$request = null,$response = null,$order=null){
        $writer = new \Zend\Log\Writer\Stream(BP . '/var/log/cegid.log');
        $logger = new \Zend\Log\Logger();
        $logger->addWriter($writer);
        $logger->info("Description:");
        $logger->info($method);

        if(is_string($order)){
            $increment_id = $order;
        } elseif ($order && $order->getIncrementId()){
            $increment_id = $order->getIncrementId();
        } else {
            $increment_id = false;
        }
        if($increment_id){
            $logger->info("Increment ID:");
            $logger->info($increment_id);
        }

        $logger->info("Request:");
        $logger->info($request);
        $logger->info("Response:");
        $logger->info($response);
        $logger->info("=================================");
    }

    // Use this function to dynamically skip execution of code via observer method

    /**
     * @param $class_name
     * @param $function_name
     * @param $array_of_data
     * @return bool
     */
    public function shouldSkipExecution($class_name,$function_name,$array_of_data){
        // Using object to manipulate the data over the observer (if needed)
        $class_name = str_replace('\\','_',$class_name);
        $obj = new \Magento\Framework\DataObject();
        $obj->setShouldSkip(false);
        $this->_eventManager->dispatch(
            'mod2_cegid_calls_skip_execution',
            [
                'should_skip' => $obj,
                'function_name' => $function_name,
                'class_name' => $class_name,
                'array_of_data' => $array_of_data
            ]);
        $will_skip = $obj->getShouldSkip();

        if($will_skip){
            if (isset($array_of_data['processing_object_id'])){
                $this->writeLog($class_name,$function_name,"will skip this execution!",$array_of_data['processing_object_id']);
            }
        }

        return $will_skip;
    }



}